{-------------------------------------------------------------------------------

The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is "GroupSortUtils.pas", released at January 12nd, 2008.

The Initial Developer of the Original Code is

  Priyatna
  Website: http://www.priyatna.org
  Email: me@priyatna.org
  Copyright (c) 2008 Priyatna
  All Rights Reserved.
        
Contributor(s):
  - Mickal Vanneufville <mickael.vanneufville@gmail.com> add custom fields support.

Description: Group and sort movies.

Known Issues: -

Last Modified: 2014-04-19 by Mickal Vanneufville

-------------------------------------------------------------------------------}

unit GroupSortUtils;

interface

uses
  Windows, Classes, SysUtils, MovieClass, NiceSideBar, DateUtils, Dialogs;

type
  TSortKind = (
    skNone, skCategory, skOrigTitle, skTransTitle, skRating, skYear, skDecade,
    skDirector, skProducer, skCountry, skLanguage, skNumber, skMedia, skMediaType,
    skDate, skBorrower, skLength, skActor, skVideoFormat, skVideoBitrate,
    skAudioFormat, skAudioBitrate, skResolution, skFramerate, skSubtitle,
    skDisk, skSize, skChecked, skColorTag, skDateWatched, skUserRating,
    skWriter, skComposer, skCertification, skURL, skFilePath, skCustomField
  );

const
  YesNo: array [Boolean] of string = ('No', 'Yes');
  DefaultSortKind: TSortKind = skNumber;
  CustomFieldStart: Integer = Ord(skCustomField);
  NoGroupValue: string = '~**~**~';

  procedure GroupAndSortMovies(List: TList; Bar: TNiceSideBar; TitleType: Integer;
    GroupKind: TSortKind; GroupReverseOrder: Boolean; GroupCFProperties: TCustomFieldProperties;
    SortKind: TSortKind; SortReverseOrder: Boolean; SortCFProperties: TCustomFieldProperties;
    var Title: string);

implementation

uses
  TransUtils, UnitMain, Math, functions_str;

//------------------------------------------------------------------------------

function SortStrAscd(List: TStringList; Index1, Index2: Integer): Integer;
begin
  //Result := CompareText(List[Index1], List[Index2]);
  Result := NatCompareText(List[Index1], List[Index2]);
end;

function SortStrDscd(List: TStringList; Index1, Index2: Integer): Integer;
begin
  //Result := -CompareText(List[Index1], List[Index2]);
  Result := -NatCompareText(List[Index1], List[Index2]);
end;

function SortIntAscd(List: TStringList; Index1, Index2: Integer): Integer;
var
  Int1, Int2: Integer;
begin
  Int1 := StrToIntDef(List[Index1], 0);
  Int2 := StrToIntDef(List[Index2], 0);
  if (Int1 = Int2)
    then Result := 0 else
  if (Int1 > Int2)
    then Result := 1
    else Result := -1;
end;

function SortIntDscd(List: TStringList; Index1, Index2: Integer): Integer;
var
  Int1, Int2: Integer;
begin
  Int1 := StrToIntDef(List[Index1], 0);
  Int2 := StrToIntDef(List[Index2], 0);
  if (Int1 = Int2)
    then Result := 0 else
  if (Int1 > Int2)
    then Result := -1
    else Result := 1;
end;

function SortRealAscd(List: TStringList; Index1, Index2: Integer): Integer;
var
  r1, r2: Extended;
begin
  r1 := StrToFloatDef(List[Index1], 0, FormatSettings);
  r2 := StrToFloatDef(List[Index2], 0, FormatSettings);
  if (r1 = r2)
    then Result := 0 else
  if (r1 > r2)
    then Result := 1
    else Result := -1;
end;

function SortRealDscd(List: TStringList; Index1, Index2: Integer): Integer;
var
  r1, r2: Extended;
begin
  r1 := StrToFloatDef(List[Index1], 0, FormatSettings);
  r2 := StrToFloatDef(List[Index2], 0, FormatSettings);
  if (r1 = r2)
    then Result := 0 else
  if (r1 > r2)
    then Result := -1
    else Result := 1;
end;

function SortDateAscd(List: TStringList; Index1, Index2: Integer): Integer;
var
  d1, d2: TDateTime;
begin
  d1 := StrToDateDef(List[Index1], 0, FormatSettings);
  d2 := StrToDateDef(List[Index2], 0, FormatSettings);
  if (d1 = d2)
    then Result := 0 else
  if (d1 > d2)
    then Result := 1
    else Result := -1;
end;

function SortDateDscd(List: TStringList; Index1, Index2: Integer): Integer;
var
  d1, d2: TDateTime;
begin
  d1 := StrToDateDef(List[Index1], 0, FormatSettings);
  d2 := StrToDateDef(List[Index2], 0, FormatSettings);
  if (d1 = d2)
    then Result := 0 else
  if (d1 > d2)
    then Result := -1
    else Result := 1;
end;

//------------------------------------------------------------------------------

procedure FlatCommaText(List: TStringList; const Txt: string; Movie: TMovie);
var
  Last, Str: string;
  i, l: Integer;
  c: Char;
  ParenCount: Integer;
begin
  if (Txt = '') then
  begin
    //List.AddObject(_('Others'), Movie);
    List.AddObject(_('< n/a >'), Movie);
    Exit;
  end;
  Str := '';
  Last := '';
  l := Length(Txt);
  i := 1;
  while (i <= l) do
  begin
    c := Txt[i];
    Inc(i);
    case c of
      ',': begin
             Str := Trim(Str);
             if (Str <> '') and (Last <> Str)
               then List.AddObject(Str, Movie);
             Last := Str;
             Str := '';
           end;
      '(': begin
             ParenCount := 1;
             while (i <= l) do
             begin
               c := Txt[i];
               Inc(i);
               case c of
                 '(': ParenCount := ParenCount + 1;
                 ')': ParenCount := ParenCount - 1;
               end;
               if (ParenCount = 0)
                 then Break;
             end;
           end;
    else
      Str := Str + c;
    end;
  end;
  Str := Trim(Str);
  if (Str <> '') and (Last <> Str)
    then List.AddObject(Str, Movie);
end;

procedure FlatCategory(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  FlatCommaText(List, Movie.strCategory, Movie);
end;

procedure FlatOriginalTitle(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(UpperCase(Copy(Movie.strOriginalTitle, 1, 1)), Movie);
end;

procedure FlatTranslatedTitle(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  if (Movie.strTranslatedTitle <> '')
    then List.AddObject(UpperCase(Copy(Movie.strTranslatedTitle, 1, 1)), Movie)
    else List.AddObject(UpperCase(Copy(Movie.strOriginalTitle, 1, 1)), Movie);
end;

procedure FlatRating(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(IntToStr(Movie.iRating), Movie);
end;

procedure FlatYear(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(IntToStr(Movie.iYear), Movie);
end;

procedure FlatDecade(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(IntToStr(Movie.iYear), Movie);
end;

procedure FlatDirector(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  FlatCommaText(List, Movie.strDirector, Movie);
end;

procedure FlatProducer(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  FlatCommaText(List, Movie.strProducer, Movie);
end;

procedure FlatCountry(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  //List.AddObject(Movie.strCountry, Movie);
  FlatCommaText(List, Movie.strCountry, Movie);
end;

procedure FlatLanguage(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  FlatCommaText(List, Movie.strLanguages, Movie);
end;

procedure FlatNumber(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(IntToStr(Movie.iNumber), Movie);
end;

procedure FlatMedia(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(Movie.strMedia, Movie);
end;

procedure FlatMediaType(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  //List.AddObject(Movie.strMediaType, Movie);
  FlatCommaText(List, Movie.strMediaType, Movie);
end;

procedure FlatDate(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(IntToStr(Movie.iDate), Movie);
end;

procedure FlatBorrower(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(Movie.strBorrower, Movie);
end;

procedure FlatLength(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(IntToStr(Movie.iLength), Movie);
end;

procedure FlatActors(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  FlatCommaText(List, Movie.strActors, Movie);
end;

procedure FlatVideoFormat(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  //List.AddObject(Movie.strVideoFormat, Movie);
  FlatCommaText(List, Movie.strVideoFormat, Movie);
end;

procedure FlatVideoBitrate(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(IntToStr(Movie.iVideoBitrate), Movie);
end;

procedure FlatAudioFormat(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  //List.AddObject(Movie.strAudioFormat, Movie);
  FlatCommaText(List, Movie.strAudioFormat, Movie);
end;

procedure FlatAudioBitrate(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(IntToStr(Movie.iAudioBitrate), Movie);
end;

procedure FlatResolution(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(Movie.strResolution, Movie);
end;

procedure FlatFramerate(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(Movie.strFramerate, Movie);
end;

procedure FlatSubtitles(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  FlatCommaText(List, Movie.strSubtitles, Movie);
end;

procedure FlatDisks(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(IntToStr(Movie.iDisks), Movie);
end;

procedure FlatSize(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(Movie.strSize, Movie);
end;

procedure FlatChecked(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(_(YesNo[Movie.bChecked]), Movie);
end;

procedure FlatColorTag(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(IntToStr(Movie.iColorTag), Movie);
end;

procedure FlatDateWatched(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(IntToStr(Movie.iDateWatched), Movie);
end;

procedure FlatUserRating(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(IntToStr(Movie.iUserRating), Movie);
end;

procedure FlatWriter(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  FlatCommaText(List, Movie.strWriter, Movie);
end;

procedure FlatComposer(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  FlatCommaText(List, Movie.strComposer, Movie);
end;

procedure FlatCertification(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  FlatCommaText(List, Movie.strCertification, Movie);
end;

procedure FlatURL(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(Movie.strURL, Movie);
end;

procedure FlatFilePath(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(Movie.strFilePath, Movie);
end;

procedure FlatCustomField(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  if CFProperties.MultiValues then
    FlatCommaText(List, Movie.CustomFields.GetFieldValue(CFProperties.FieldTag, False), Movie)
  else
    List.AddObject(Movie.CustomFields.GetFieldValue(CFProperties.FieldTag, False), Movie);
end;

//------------------------------------------------------------------------------

procedure AddOnceCategory(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(Movie.strCategory, Movie);
end;

procedure AddOnceOriginalTitle(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(FormMain.GetMovieTitle(Movie, ttOriginal), Movie);
end;

procedure AddOnceTranslatedTitle(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(FormMain.GetMovieTitle(Movie, ttTranslated), Movie);
end;

procedure AddOnceMediaTitle(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(FormMain.GetMovieTitle(Movie, ttMedia), Movie);
end;

procedure AddOnceRating(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(IntToStr(Movie.iRating), Movie);
end;

procedure AddOnceYear(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(IntToStr(Movie.iYear), Movie);
end;

procedure AddOnceDecade(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(IntToStr(Movie.iYear), Movie);
end;

procedure AddOnceDirector(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(Movie.strDirector, Movie);
end;

procedure AddOnceProducer(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(Movie.strProducer, Movie);
end;

procedure AddOnceCountry(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(Movie.strCountry, Movie);
end;

procedure AddOnceLanguage(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(Movie.strLanguages, Movie);
end;

procedure AddOnceNumber(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(IntToStr(Movie.iNumber), Movie);
end;

procedure AddOnceMedia(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(Movie.strMedia, Movie);
end;

procedure AddOnceMediaType(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(Movie.strMediaType, Movie);
end;

procedure AddOnceDate(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(IntToStr(Movie.iDate), Movie);
end;

procedure AddOnceBorrower(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(Movie.strBorrower, Movie);
end;

procedure AddOnceLength(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(IntToStr(Movie.iLength), Movie);
end;

procedure AddOnceActors(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(Movie.strActors, Movie);
end;

procedure AddOnceVideoFormat(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(Movie.strVideoFormat, Movie);
end;

procedure AddOnceVideoBitrate(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(IntToStr(Movie.iVideoBitrate), Movie);
end;

procedure AddOnceAudioFormat(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(Movie.strAudioFormat, Movie);
end;

procedure AddOnceAudioBitrate(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(IntToStr(Movie.iAudioBitrate), Movie);
end;

procedure AddOnceResolution(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(Movie.strResolution, Movie);
end;

procedure AddOnceFramerate(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(Movie.strFramerate, Movie);
end;

procedure AddOnceSubtitles(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(Movie.strSubtitles, Movie);
end;

procedure AddOnceDisks(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(IntToStr(Movie.iDisks), Movie);
end;

procedure AddOnceSize(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(Movie.strSize, Movie);
end;

procedure AddOnceChecked(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(_(YesNo[Movie.bChecked]), Movie);
end;

procedure AddOnceColorTag(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(IntToStr(Movie.iColorTag), Movie);
end;

procedure AddOnceDateWatched(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(IntToStr(Movie.iDateWatched), Movie);
end;

procedure AddOnceUserRating(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(IntToStr(Movie.iUserRating), Movie);
end;

procedure AddOnceWriter(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(Movie.strWriter, Movie);
end;

procedure AddOnceComposer(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(Movie.strComposer, Movie);
end;

procedure AddOnceCertification(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(Movie.strCertification, Movie);
end;

procedure AddOnceURL(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(Movie.strURL, Movie);
end;

procedure AddOnceFilePath(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(Movie.strFilePath, Movie);
end;

procedure AddOnceCustomField(List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
begin
  List.AddObject(Movie.CustomFields.GetFieldValue(CFProperties.FieldTag, False), Movie);
end;

//------------------------------------------------------------------------------


procedure GroupString(const Item1, Item2: string; var SameGroup: Boolean; var NewGroup: string);
begin
  SameGroup := SameText(Item1, Item2);
  if not SameGroup then
  begin
    if (Item2 <> '')
      then NewGroup := Item2
      else NewGroup := _('< n/a >');
  end;
end;

procedure GroupInteger(const Item1, Item2: string; var SameGroup: Boolean; var NewGroup: string);
var
  i1, i2: Integer;
begin
  i1 := StrToIntDef(Item1, Integer($FFFFFFFF));
  i2 := StrtoIntDef(Item2, Integer($FFFFFFFF));
  SameGroup := (Item1 <> NoGroupValue) and (i1 = i2);
  if not SameGroup then
  begin
    if (i2 = Integer($FFFFFFFF))
      then NewGroup := _('< n/a >')
      else NewGroup := IntToStr(i2);
  end;
end;

procedure GroupReal(const Item1, Item2: string; var SameGroup: Boolean; var NewGroup: string);
var
  r1, r2: Extended;
begin
  r1 := StrToFloatDef(Item1, NaN, FormatSettings);
  r2 := StrToFloatDef(Item2, Nan, FormatSettings);
  SameGroup := (Item1 <> NoGroupValue) and (r1 = r2);
  if not SameGroup then
  begin
    if (IsNan(r2))
      then NewGroup := _('< n/a >')
      else NewGroup := Format('#0.000', [r2]);
  end;
end;

procedure GroupDateStr(const Item1, Item2: string; var SameGroup: Boolean; var NewGroup: string);
var
  d1, d2: TDateTime;
begin
  d1 := StrToDateDef(Item1, 0, FormatSettings);
  d2 := StrToDateDef(Item2, 0, FormatSettings);
  SameGroup := (Item1 <> NoGroupValue) and (d1 = d2);
  if not SameGroup then
  begin
    if (d2 = 0)
      then NewGroup := _('< n/a >')
      else NewGroup := FormatDateTime(FormMain.ConfDateFormat, d2);
  end;
end;

procedure GroupAlphabet(const Item1, Item2: string; var SameGroup: Boolean; var NewGroup: string);
begin
  SameGroup := Item1 = Item2;
  if not SameGroup then
  begin
    if (Item2 <> '')
      then NewGroup := _('Letter') + ' ' + Item2
      else NewGroup := _('< n/a >');
  end;
end;

procedure GroupPeople(const Item1, Item2: string; var SameGroup: Boolean; var NewGroup: string);
begin
  SameGroup := (Item1 <> NoGroupValue) and ((UpperCase(Copy(Item1, 1, 1)) = UpperCase(Copy(Item2, 1, 1))));
  if not SameGroup then
  begin
    if (Item2 <> '')
      then NewGroup := _('Letter') + ' ' + UpperCase(Copy(Item2, 1, 1))
      else NewGroup := _('< n/a >');
  end;
end;

procedure GroupRating(const Item1, Item2: string; var SameGroup: Boolean; var NewGroup: string);
var
  i1, i2: Integer;
begin
  i1 := StrToIntDef(Item1, 0) div 10;
  i2 := StrtoIntDef(Item2, 0) div 10;
  SameGroup := (Item1 <> NoGroupValue) and (i1 = i2);
  if not SameGroup then
  begin
    if (i2 <= 0)
      then NewGroup := _('< n/a >')
      else NewGroup := _('Rating') + ' ' + IntToStr(i2);
  end;
end;

procedure GroupYear(const Item1, Item2: string; var SameGroup: Boolean; var NewGroup: string);
var
  i1, i2: Integer;
begin
  i1 := StrToIntDef(Item1, 0);
  i2 := StrtoIntDef(Item2, 0);
  SameGroup := (Item1 <> NoGroupValue) and (i1 = i2);
  if not SameGroup then
  begin
    if (i2 <= 0)
      then NewGroup := _('< n/a >')
      else NewGroup := _('Year') + ' ' + IntToStr(i2);
  end;
end;

procedure GroupDecade(const Item1, Item2: string; var SameGroup: Boolean; var NewGroup: string);
var
  i1, i2: Integer;
begin
  i1 := StrToIntDef(Item1, 0) div 10;
  i2 := StrtoIntDef(Item2, 0) div 10;
  SameGroup := (Item1 <> NoGroupValue) and (i1 = i2);
  if not SameGroup then
  begin
    if (i2 <= 0)
      then NewGroup := _('< n/a >')
      else NewGroup := _('Decade') + ' ' + IntToStr(i2 * 10);
  end;
end;

procedure GroupNumber(const Item1, Item2: string; var SameGroup: Boolean; var NewGroup: string);
var
  i1, i2: Integer;
begin
  i1 := StrToIntDef(Item1, 0) div 50;
  i2 := StrtoIntDef(Item2, 0) div 50;
  SameGroup := (Item1 <> NoGroupValue) and (i1 = i2);
  if not SameGroup
    then NewGroup := _('Number') + ' ' + IntToStr(i2 * 50) + ' - ' + IntToStr((i2 * 50) + 49);
end;

(*procedure GroupDate(const Item1, Item2: string; var SameGroup: Boolean; var NewGroup: string);
var
  i1, i2: Integer;
begin
  i1 := YearOf(StrToIntDef(Item1, 0));
  i2 := YearOf(StrtoIntDef(Item2, 0));
  SameGroup := (Item1 <> NoGroupValue) and (i1 = i2);
  if not SameGroup
    then NewGroup := _('Year') + ' ' + IntToStr(i2);
end;*)

procedure GroupDate(const Item1, Item2: string; var SameGroup: Boolean; var NewGroup: string);
var
  d1, d2: TDateTime;
begin
  d1 := StrToIntDef(Item1, 0);
  d2 := StrToIntDef(Item2, 0);
  SameGroup := (Item1 <> NoGroupValue) and (d1 = d2);
  if not SameGroup then
  begin
    if (d2 = 0)
      then NewGroup := _('< n/a >')
      else NewGroup := FormatDateTime(FormMain.ConfDateFormat, d2);
  end;
end;

procedure GroupLength(const Item1, Item2: string; var SameGroup: Boolean; var NewGroup: string);
var
  i1, i2: Integer;
begin
  i1 := StrToIntDef(Item1, 0) div 10;
  i2 := StrtoIntDef(Item2, 0) div 10;
  SameGroup := (Item1 <> NoGroupValue) and (i1 = i2);
  if not SameGroup then
  begin
    if (i2 <= 0)
      then NewGroup := _('< n/a >')
      else NewGroup := IntToStr(i2 * 10) + ' ' + _('Minutes');
  end;
end;

procedure GroupVideoBitrate(const Item1, Item2: string; var SameGroup: Boolean; var NewGroup: string);
var
  i1, i2: Integer;
begin
  i1 := StrToIntDef(Item1, 0);
  i2 := StrtoIntDef(Item2, 0);
  SameGroup := (Item1 <> NoGroupValue) and (i1 = i2);
  if not SameGroup then
  begin
    if (i2 <= 0)
      then NewGroup := _('< n/a >')
      else NewGroup := IntToStr(i2) + ' ' + _('Kbps');
  end;
end;

procedure GroupAudioBitrate(const Item1, Item2: string; var SameGroup: Boolean; var NewGroup: string);
var
  i1, i2: Integer;
begin
  i1 := StrToIntDef(Item1, 0);
  i2 := StrtoIntDef(Item2, 0);
  SameGroup := (Item1 <> NoGroupValue) and (i1 = i2);
  if not SameGroup then
  begin
    if (i2 <= 0)
      then NewGroup := _('< n/a >')
      else NewGroup := IntToStr(i2) + ' ' + _('Kbps');
  end;
end;

procedure GroupDisks(const Item1, Item2: string; var SameGroup: Boolean; var NewGroup: string);
var
  i1, i2: Integer;
begin
  i1 := StrToIntDef(Item1, 0);
  i2 := StrtoIntDef(Item2, 0);
  SameGroup := (Item1 <> NoGroupValue) and (i1 = i2);
  if not SameGroup then
  begin
    if (i2 <= 0)
      then NewGroup := _('< n/a >') else
    if (i2 = 1)
      then NewGroup := _('1 Disc')
      else NewGroup := IntToStr(i2) + ' ' + _('Discs');
  end;
end;


//------------------------------------------------------------------------------


type
  TFlatProc = procedure (List: TStringList; Movie: TMovie; CFProperties: TCustomFieldProperties);
  TGroupProc = procedure (const Item1, Item2: string; var SameGroup: Boolean; var NewGroup: string);

  TGroupRec = record
    GroupAddProc: TFlatProc;
    GroupSortProc: TStringListSortCompare;
    GroupProc: TGroupProc;
    GroupCFProperties: TCustomFieldProperties;
    OrderAddProc: TFlatproc;
    OrderSortProc: TStringListSortCompare;
    OrderCFProperties: TCustomFieldProperties;
    Title: string;
  end;

const
  FlatProcTables: array [TSortKind] of TFlatProc = (
    nil, FlatCategory, FlatOriginalTitle, FlatTranslatedTitle, FlatRating, FlatYear, FlatDecade,
    FlatDirector, FlatProducer, FlatCountry, FlatLanguage, FlatNumber,
    FlatMedia, FlatMediaType, FlatDate, FlatBorrower, FlatLength, FlatActors,
    FlatVideoFormat, FlatVideoBitrate, FlatAudioFormat, FlatAudioBitrate,
    FlatResolution, FlatFramerate, FlatSubtitles, FlatDisks,
    FlatSize, FlatChecked, FlatColorTag, FlatDateWatched, FlatUserRating,
    FlatWriter, FlatComposer, FlatCertification, FlatURL, FlatFilePath,
    FlatCustomField);

  AddOnceProcTables: array [TSortKind] of TFlatProc = (
    nil, AddOnceCategory, AddOnceOriginalTitle, AddOnceTranslatedTitle, AddOnceRating, AddOnceYear, AddOnceDecade,
    AddOnceDirector, AddOnceProducer, AddOnceCountry, AddOnceLanguage, AddOnceNumber,
    AddOnceMedia, AddOnceMediaType, AddOnceDate, AddOnceBorrower, AddOnceLength, AddOnceActors,
    AddOnceVideoFormat, AddOnceVideoBitrate, AddOnceAudioFormat, AddOnceAudioBitrate,
    AddOnceResolution, AddOnceFramerate, AddOnceSubtitles, AddOnceDisks,
    AddOnceSize, AddOnceChecked, AddOnceColorTag, AddOnceDateWatched, AddOnceUserRating,
    AddOnceWriter, AddOnceComposer, AddOnceCertification, AddOnceURL, AddOnceFilePath,
    AddOnceCustomField);

  SortAscProcTables: array [TSortKind] of TStringListSortCompare = (
    nil, SortStrAscd, SortStrAscd, SortStrAscd, SortIntAscd, SortIntAscd, SortIntAscd,
    SortStrAscd, SortStrAscd, SortStrAscd, SortStrAscd, SortIntAscd,
    SortStrAscd, SortStrAscd, SortIntAscd, SortStrAscd, SortIntAscd, SortStrAscd,
    SortStrAscd, SortIntAscd, SortStrAscd, SortIntAscd,
    SortStrAscd, SortStrAscd, SortStrAscd, SortIntAscd,
    SortStrAscd, SortStrAscd, SortIntAscd, SortIntAscd, SortIntAscd,
    SortStrAscd, SortStrAscd, SortStrAscd, SortStrAscd, SortStrAscd,
    SortStrAscd);

  SortDscProcTables: array [TSortKind] of TStringListSortCompare = (
    nil, SortStrDscd, SortStrDscd, SortStrDscd, SortIntDscd, SortIntDscd, SortIntDscd,
    SortStrDscd, SortStrDscd, SortStrDscd, SortStrDscd, SortIntDscd,
    SortStrDscd, SortStrDscd, SortIntDscd, SortStrDscd, SortIntDscd, SortStrDscd,
    SortStrDscd, SortIntDscd, SortStrDscd, SortIntDscd,
    SortStrDscd, SortStrDscd, SortStrDscd, SortIntDscd,
    SortStrDscd, SortStrDscd, SortIntDscd, SortIntDscd, SortIntDscd,
    SortStrDscd, SortStrDscd, SortStrDscd, SortStrDscd, SortStrDscd,
    SortStrDscd);

  GroupProcTables: array [TSortKind] of TGroupProc = (
    nil, GroupString, GroupAlphabet, GroupAlphabet, GroupRating, GroupYear, GroupDecade,
    GroupPeople, GroupPeople, GroupString, GroupString, GroupNumber,
    GroupString, GroupString, GroupDate, GroupString, GroupLength, GroupPeople,
    GroupString, GroupVideoBitrate, GroupString, GroupAudioBitrate,
    GroupString, GroupString, GroupString, GroupDisks,
    GroupString, GroupString, GroupInteger, GroupDate, GroupRating,
    GroupPeople, GroupPeople, GroupString, GroupString, GroupString,
    GroupString);

  GroupTitleTables: array [TSortKind] of string = (
    '', 'Category', 'Original Title', 'Translated Title', 'Rating', 'Year', 'Decade',
    'Director', 'Producer', 'Country', 'Language', 'Number',
    'Media', 'Media Type', 'Date', 'Borrower', 'Length', 'Actors',
    'Video Format', 'Video Bitrate', 'Audio Format', 'Audio Bitrate',
    'Resolution', 'Framerate', 'Subtitles', 'Disks',
    'Size', 'Checked', 'Color Tag', 'Date Watched', 'User Rating',
    'Writer', 'Composer', 'Certification', 'URL', 'File Path',
    'Custom Field');

procedure GetGroupRec(GroupKind: TSortKind; GroupReverseOrder: Boolean; GroupCFProperties: TCustomFieldProperties;
  SortKind: TSortKind; SortReverseOrder: Boolean; SortCFProperties: TCustomFieldProperties; var Info: TGroupRec);
var
  sk: TSortKind;
begin
  Info.GroupAddProc := FlatProcTables[GroupKind];

  if GroupReverseOrder
    then Info.GroupSortProc := SortDscProcTables[GroupKind]
    else Info.GroupSortProc := SortAscProcTables[GroupKind];
  if (GroupKind = skCustomField) then
    if (GroupCFProperties.FieldType = ftInteger) then
      if GroupReverseOrder then
        Info.GroupSortProc := SortIntDscd
      else
        Info.GroupSortProc := SortIntAscd
    else if (GroupCFProperties.FieldType = ftReal) then
      if GroupReverseOrder then
        Info.GroupSortProc := SortRealDscd
      else
        Info.GroupSortProc := SortRealAscd
    else if (GroupCFProperties.FieldType = ftDate) then
      if GroupReverseOrder then
        Info.GroupSortProc := SortDateDscd
      else
        Info.GroupSortProc := SortDateAscd;

  Info.GroupProc := GroupProcTables[GroupKind];
  if (GroupKind = skCustomField) then
    if (GroupCFProperties.FieldType = ftInteger) then
      Info.GroupProc := GroupInteger
    else if (GroupCFProperties.FieldType = ftReal) then
      Info.GroupProc := GroupReal
    else if (GroupCFProperties.FieldType = ftDate) then
      Info.GroupProc := GroupDateStr;

  if (SortKind = skNone)
    then sk := DefaultSortKind
    else sk := SortKind;

  Info.GroupCFProperties := GroupCFProperties;

  Info.OrderAddProc := AddOnceProcTables[sk];

  if SortReverseOrder
    then Info.OrderSortProc := SortDscProcTables[sk]
    else Info.OrderSortProc := SortAscProcTables[sk];
  if (SortKind = skCustomField) then
    if (SortCFProperties.FieldType = ftInteger) then
      if SortReverseOrder then
        Info.OrderSortProc := SortIntDscd
      else
        Info.OrderSortProc := SortIntAscd
    else if (SortCFProperties.FieldType = ftReal) then
      if SortReverseOrder then
        Info.OrderSortProc := SortRealDscd
      else
        Info.OrderSortProc := SortRealAscd
    else if (SortCFProperties.FieldType = ftDate) then
      if SortReverseOrder then
        Info.OrderSortProc := SortDateDscd
      else
        Info.OrderSortProc := SortDateAscd;

  Info.OrderCFProperties := SortCFProperties;

  Info.Title := '';
  if (GroupKind <> skNone) then
    if (GroupKind <> skCustomField) then
      Info.Title := _('Group by') + ' ' + _(GroupTitleTables[GroupKind]) + ', '
    else
      Info.Title := _('Group by') + ' ' + GroupCFProperties.FieldName + ', ';
  if (SortKind <> skCustomField) then
    Info.Title := Info.Title + _('Order by') + ' ' + _(GroupTitleTables[sk])
  else
    Info.Title := Info.Title + _('Order by') + ' ' + SortCFProperties.FieldName;
end;

procedure GroupAndSortMovies(List: TList; Bar: TNiceSideBar; TitleType: Integer;
  GroupKind: TSortKind; GroupReverseOrder: Boolean; GroupCFProperties: TCustomFieldProperties;
  SortKind: TSortKind; SortReverseOrder: Boolean; SortCFProperties: TCustomFieldProperties;
  var Title: string);
label
  Done;
var
  Info: TGroupRec;
  t: TStringList;
  x, y: Integer;
  Section: TSideBarItem;
  SameGroup: Boolean;
  NewGroup: string;
  TitleProc: TFlatProc;
  IsPeople: Boolean;
  CurrMovies, CurrPerson: string;
  m: TStringList;
begin
  Bar.BeginUpdate;
  t := TStringList.Create;
  try
    if (Bar.Tag = 1) then
    begin
      for x := 0 to Bar.Items.Count-1 do
      begin
        m := TStringList(Bar.Items[x].Tag);
        if Assigned(m)
          then m.Free;
      end;
    end;  
    Bar.Items.Clear;
    Bar.Tag := 0;
    if (List.Count = 0)
      then goto Done;
    GetGroupRec(GroupKind, GroupReverseOrder, GroupCFProperties,
      SortKind, SortReverseOrder, SortCFProperties, Info);
    Title := Info.Title;
    if Assigned(Info.GroupAddProc) then
    begin
      for x := 0 to List.Count-1
        do Info.GroupAddProc(t, TMovie(List[x]), GroupCFProperties);
      if Assigned(Info.GroupSortProc)
        then t.CustomSort(Info.GroupSortProc)
        else t.Sort;
      Info.GroupProc(NoGroupValue, t[0], SameGroup, NewGroup);
      Section := Bar.Items.Add;
      Section.Caption := NewGroup;
      Section.Expanded := False;
      Section.Items.AddObject(t[0], t.Objects[0]);
      for x := 1 to t.Count-1 do
      begin
        Info.GroupProc(t[x-1], t[x], SameGroup, NewGroup);
        if not SameGroup then
        begin
          Section := Bar.Items.Add;
          Section.Caption := NewGroup;
          Section.Expanded := False;
        end;
        Section.Items.AddObject(t[x], t.Objects[x]);
      end;
    end else
    begin
      Section := Bar.Items.Add;
      Section.Caption := _('All Movies');
      for x := 0 to List.Count-1
        do Section.Items.AddObject('', TObject(List[x]));
    end;
    IsPeople := @Info.GroupProc = @GroupPeople;
    if IsPeople then
    begin
      Bar.Tag := 1;
      for x := 0 to Bar.Items.Count-1 do
      begin
        Section := Bar.Items[x];
        if (Section.Items.Count = 0)
          then Continue;
        t.Clear;
        t.Assign(Section.Items);
        if SortReverseOrder
          then t.CustomSort(SortStrDscd)
          else t.Sort;
        Section.Items.Clear;
        m := TStringList.Create;
        Section.Tag := Integer(m);
        CurrPerson := t[0];
        CurrMovies := IntToStr(Integer(t.Objects[0]));
        for y := 1 to t.Count-1 do
        begin
          if (t[y] <> CurrPerson) then
          begin
            Section.Items.Add(CurrPerson);
            m.Add(CurrMovies);
            CurrPerson := t[y];
            CurrMovies := '';
          end;
          if (CurrMovies <> '')
            then CurrMovies := CurrMovies + ',';
          CurrMovies := CurrMovies + IntToStr(Integer(t.Objects[y]));
        end;
        if (CurrMovies <> '') then
        begin
          Section.Items.Add(CurrPerson);
          m.Add(CurrMovies);
        end;

        Section.Caption := Section.Caption + ' (' + IntToStr(Section.Items.Count) + ')';

      end;

    end else
    begin
      if (TitleType = 0) then
        TitleProc := AddOnceOriginalTitle
      else if (TitleType = 1) then
        TitleProc := AddOnceTranslatedTitle
      else
        TitleProc := AddOnceMediaTitle;
      for x := 0 to Bar.Items.Count-1 do
      begin
        Section := Bar.Items[x];
        Section.Caption := Section.Caption + ' (' + IntToStr(Section.Items.Count) + ')';
        t.Clear;
        for y := 0 to Section.Items.Count-1
          do Info.OrderAddProc(t, TMovie(Section.Items.Objects[y]), SortCFProperties);
        if Assigned(Info.OrderSortProc)
          then t.CustomSort(Info.OrderSortProc)
          else t.Sort;
        Section.Items.Clear;
        for y := 0 to t.Count-1
          do TitleProc(Section.Items, TMovie(t.Objects[y]), SortCFProperties);
      end;
    end;
    Done:
  finally
    Bar.EndUpdate;
    t.Free;
  end;
end;

end.

